<div class="chatlog__embed-thumbnail-container">
    <a class="chatlog__embed-thumbnail-link" href="{{EMBED_THUMBNAIL}}">
    <img class="chatlog__embed-thumbnail" src="{{EMBED_THUMBNAIL}}">
</a>
</div>