
<div id="meta-popout-{{USER_ID}}" class="meta-popout">
    <div class="meta__header">
         <img src="{{USER_AVATAR}}" alt="Avatar">
    </div>
    <div class="meta__description">
        {{DISPLAY}}
        <div class="meta__details">
            <div class="meta__user">{{USERNAME}}</div>
            <div class="meta__discriminator">{{DISCRIMINATOR}}</div>
            {{BOT}}
        </div>
        <div class="meta__divider-2"></div>
        <div class="meta__field">
            <div class="meta__title">Member Since</div>
            <div class="meta__value"><img src="{{DISCORD_ICON}}"/> {{CREATED_AT}} <div class="meta__divider"></div> <img src="{{GUILD_ICON}}" class="meta__img-border"/> {{JOINED_AT}}</div>
        </div>
        <div class="meta__field">
            <div class="meta__title">Member ID</div>
            <div class="meta__value">{{MEMBER_ID}}</div>
        </div>
        <div class="meta__field">
            <div class="meta__title">Message Count</div>
            <div class="meta__value">{{MESSAGE_COUNT}}</div>
        </div>
    </div>
</div>
